#! /usr/bin/env python
#
# ======================================================================== 
#
# Python module for SBIG CCD camera - python test code
# Copyright (C) 2006 Laurent Le Guillou <llg@lpnhep.in2p3.fr>
# Mercator Telescope, C/A. de Abreu 70, 38700 Santa Cruz de La Palma, Spain
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301  USA
#
# ========================================================================

import sys
import os
import time
import sbig
import numarray
import pyfits

sbig.open_driver()
devices = ()
# while not(devices):
devices = sbig.query_usb()    # return list of USB cameras
# [TODO: BUG: works only once]
print "devices = ", devices

sbig.open_device(sbig.DEV_USB)      

tries=5
camera_type = sbig.NO_CAMERA
for i in xrange(tries):
    try:
        camera_type = sbig.establish_link()
        break
    except sbig.SBIGDriverError:
        pass

if (camera_type == sbig.NO_CAMERA):
    print >>sys.stderr, "Cannot establish link with the camera."
    sys.exit(2)

ccdinfo = sbig.get_ccd_info()

print ccdinfo

# Read all the control parameters (DCP_*)
parameter_names = [n for n in dir(sbig) if n[0:3]=='DCP']
for p in parameter_names:
    print p, sbig.get_driver_control(eval('sbig.' + p))

# Test only (useless)
# sbig.set_driver_control(sbig.DCP_USB_FIFO_SIZE, 8192) # divide by 2
# sbig.set_driver_control(sbig.DCP_USB_FIFO_SIZE, 4096) # divide by 2

# sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 1) 
# sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 0) 
sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 1) 

# Read all the control parameters (DCP_*)
parameter_names = [n for n in dir(sbig) if n[0:3]=='DCP']
for p in parameter_names:
    print p, sbig.get_driver_control(eval('sbig.' + p))

# ================ Temperature regulation ==========================

setpoint = 15

# Enable temperature regulation
sbig.set_temperature_regulation(sbig.REGULATION_ON, setpoint_celsius=setpoint)

# Regulation will be "frozen" during readout to minimize readout noise.
# (Freeze will start at the first readout_line command and stop
# when the end_readout command will be sent.)
sbig.set_temperature_regulation(sbig.REGULATION_ENABLE_AUTOFREEZE)

# Wait until the temperature reach the setpoint
temp_status = sbig.query_temperature_status()
while abs(temp_status['ccd_celsius'] - temp_status['setpoint_celsius']) > 0.2:
    print "ccd_celsius = ", temp_status['ccd_celsius']
    print "setpoint = ", temp_status['setpoint_celsius']
    time.sleep(5)
    temp_status = sbig.query_temperature_status()

# ================ First exposure: 0s (BIAS equiv) =================

sbig.start_exposure(ccd=sbig.CCD_IMAGING, exposure=0.0, shutter=sbig.SC_CLOSE_SHUTTER)
while (sbig.query_command_status() == sbig.STATUS_INTEGRATING):
    time.sleep(.1)
sbig.end_exposure(ccd=sbig.CCD_IMAGING)

mode = 0
fullheight = ccdinfo['modes'][mode]['height']
fullwidth = ccdinfo['modes'][mode]['width']
height = fullheight
width = fullwidth
top = 0
left = 0

sbig.start_readout(ccd=sbig.CCD_IMAGING, mode=mode, top=top, left=left,
                   height=height, width=width)
bias = sbig.readout_lines(ccd=sbig.CCD_IMAGING, mode=mode,
                          start=left, length=width, lines=height);
sbig.end_readout(ccd=sbig.CCD_IMAGING)

print bias.shape
print "BIAS"
print "min = ", bias.min(), "  max = ", bias.max()
print "mean = ", bias.mean(), "  rms = ", bias.stddev()

filename = "T-%d-bias.fits" % setpoint
fitsobj = pyfits.HDUList()
hdu = pyfits.PrimaryHDU()
# scale the data to Int16 with user specified bscale/bzero
# no other operation should manipulate the data after this
# hdu.data = bias
hdu.data = bias*1.0  # seems needed for the next operation
hdu.scale('Int16', '', bzero=32768)
fitsobj.append(hdu)
try:
    os.unlink(filename)
except OSError:
    pass
fitsobj.writeto(filename)

print "BIAS"
print "min = ", bias.min(), "  max = ", bias.max()
print "mean = ", bias.mean(), "  rms = ", bias.stddev()

for exposure in [10, 50, 100]:
    # ================ Second exposure: (DARK equiv) ==============

    sbig.start_exposure(ccd=sbig.CCD_IMAGING, exposure=exposure, shutter=sbig.SC_CLOSE_SHUTTER)
    while (sbig.query_command_status() == sbig.STATUS_INTEGRATING):
        time.sleep(1)
    sbig.end_exposure(ccd=sbig.CCD_IMAGING)
        
    mode = 0
    fullheight = ccdinfo['modes'][mode]['height']
    fullwidth = ccdinfo['modes'][mode]['width']
    height = fullheight
    width = fullwidth
    top = 0
    left = 0
    
    sbig.start_readout(ccd=sbig.CCD_IMAGING, mode=mode, top=top, left=left,
                       height=height, width=width)
    dark = sbig.readout_lines(ccd=sbig.CCD_IMAGING, mode=mode,
                              start=left, length=width, lines=height);
    sbig.end_readout(ccd=sbig.CCD_IMAGING)
    
    print dark.shape
    print "DARK"
    print "min = ", dark.min(), "  max = ", dark.max()
    print "mean = ", dark.mean(), "  rms = ", dark.stddev()
    
    filename = "T-%d-exp-%d-dark.fits" % (setpoint, exposure)
    fitsobj = pyfits.HDUList()
    hdu = pyfits.PrimaryHDU()
    # scale the data to Int16 with user specified bscale/bzero
    # no other operation should manipulate the data after this
    # hdu.data = dark
    hdu.data = dark*1.0  # seems needed for the next operation
    hdu.scale('Int16', '', bzero=32768)
    fitsobj.append(hdu)
    try:
        os.unlink(filename)
    except OSError:
        pass
    fitsobj.writeto(filename)
    print "DARK"
    print "min = ", dark.min(), "  max = ", dark.max()
    print "mean = ", dark.mean(), "  rms = ", dark.stddev()

    # ==================================================================

    dark_bias = dark*1.0 - bias*1.0

    print "DARK-BIAS"
    print "min = ", dark_bias.min(), "  max = ", dark_bias.max()
    print "mean = ", dark_bias.mean(), "  rms = ", dark_bias.stddev()
    
    filename = "T-%d-exp-%d-dark-bias.fits" % (setpoint, exposure)
    fitsobj = pyfits.HDUList()
    hdu = pyfits.PrimaryHDU()
    # scale the data to Int16 with user specified bscale/bzero
    # no other operation should manipulate the data after this
    # hdu.data = dark
    hdu.data = dark_bias
    fitsobj.append(hdu)
    try:
        os.unlink(filename)
    except OSError:
        pass
    fitsobj.writeto(filename)

# ==================================================================

sbig.close_device()
sbig.close_driver()

# ==================================================================
