#! /usr/bin/env python
#
# ======================================================================== 
#
# Python module for SBIG CCD camera - python test code
# Copyright (C) 2006 Laurent Le Guillou <laurentl@ster.kuleuven.be>
# Mercator Telescope, C/A. de Abreu 70, 38700 Santa Cruz de La Palma, Spain
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301  USA
#
# ========================================================================
#
# Some test results:
# ------------------
#
# 'Oneshot': full frame, not binned  (mode=0): readout 2.5s
# 'Oneshot': full frame, binning 3x3 (mode=2): readout 1.2s
# 'Oneshot': full frame, binning 3x3 (mode=8): readout 1.9s
# 'Oneshot': full frame, binning 9x9 (mode=9): readout 1.0s
#
# ('Oneshot' means: using readout_lines(lines=height) instead
# of reading line by line with readout_line().)
#
# ========================================================================

import os
import time
import datetime
import sbig
import numarray
import pyfits

sbig.open_driver()
devices = ()
# while not(devices):
devices = sbig.query_usb()    # return list of USB cameras
# [TODO: BUG: works only once]
print "devices = ", devices

sbig.open_device(sbig.DEV_USB)      

tries=5
camera_type = sbig.NO_CAMERA
for i in xrange(tries):
    try:
        camera_type = sbig.establish_link()
        break
    except sbig.SBIGDriverError:
        pass

if (camera_type == sbig.NO_CAMERA):
    print >>sys.stderr, "Cannot establish link with the camera."
    sys.exit(2)

ccdinfo = sbig.get_ccd_info()

print ccdinfo

# Read all the control parameters (DCP_*)
parameter_names = [n for n in dir(sbig) if n[0:3]=='DCP']
for p in parameter_names:
    print p, sbig.get_driver_control(eval('sbig.' + p))

# Test only (useless)
# sbig.set_driver_control(sbig.DCP_USB_FIFO_SIZE, 8192) # divide by 2
# sbig.set_driver_control(sbig.DCP_USB_FIFO_SIZE, 4096) # divide by 2

# sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 1) 
# sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 0) 
sbig.set_driver_control(sbig.DCP_USB_FIFO_ENABLE, 1) 

# Read all the control parameters (DCP_*)
parameter_names = [n for n in dir(sbig) if n[0:3]=='DCP']
for p in parameter_names:
    print p, sbig.get_driver_control(eval('sbig.' + p))

nexp = 10
top_start = datetime.datetime.utcnow()

for i in xrange(nexp):

    sbig.start_exposure(ccd=sbig.CCD_IMAGING, exposure=0.1)
    while (sbig.query_command_status() == sbig.STATUS_INTEGRATING):
        time.sleep(.1)
    sbig.end_exposure(ccd=sbig.CCD_IMAGING)

    mode = 0
    # mode = 2  # 3x3
    # mode = 9  # 9x9
    # mode = 8
    fullheight = ccdinfo['modes'][mode]['height']
    fullwidth = ccdinfo['modes'][mode]['width']
    height = fullheight
    width = fullwidth
    top = 0
    left = 0
    # height=400
    # width=500
    # top=100
    # left=100

    sbig.start_readout(ccd=sbig.CCD_IMAGING, mode=mode, top=top, left=left,
                       height=height, width=width)

    oneshot = True
    if (oneshot):
        image = sbig.readout_lines(ccd=sbig.CCD_IMAGING, mode=mode,
                                   start=left, length=width, lines=height);
    
    else:
        image = numarray.zeros(shape=(height, width))

        for line in xrange(height):
            # you should repeat the next line to read every row
            tries = 5
            while (tries > 0):
                try:
                    l = sbig.readout_line(ccd=sbig.CCD_IMAGING, mode=mode,
                                          start=left, length=width)
                    break
                except sbig.SBIGDriverError, msg:
                    print >>sys.stderr, ("line [%d]" % line), msg
                    tries -= 1
    
            if (tries == 0):
                print >>sys.stderr, "error: timeout (5 times)"
                sys.exit(2)
    
            image[line] = l
        # end loop

        sbig.end_readout(ccd=sbig.CCD_IMAGING)

    # end if

# end loop on shots

# print image.shape
# print "mean = ", image.mean(), "  rms = ", image.stddev()

top_end = datetime.datetime.utcnow()

delta = top_end - top_start
print (delta.seconds + delta.microseconds/(1.0e6)) / nexp

filename = "out.fits"
fitsobj = pyfits.HDUList()
hdu = pyfits.PrimaryHDU()
hdu.data = image
fitsobj.append(hdu)
try:
    os.unlink(filename)
except OSError:
    pass
fitsobj.writeto(filename)

sbig.close_device()
sbig.close_driver()

# ========================================================================
