#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include <sbigudrv.h>

#include <fitsio.h>


/*
 * Compiling:
 * 
 *   gcc -c testsbig.c
 *   gcc -o testsbig testsbig.o -lsbigudrv -L/usr/local/lib -lm -lcfitsio
 *
 */

#define HEIGHT 1020
#define WIDTH  1530

int main(int argc, char *argv[]) {

  int i, j;
  int row;
  unsigned short sts;
  unsigned short img[HEIGHT*WIDTH];
  fitsfile *fptr;
  long naxes[2] = {WIDTH, HEIGHT};
  int naxis = 2;
  short firstpixel = 1;
  int ok;
  int status = 0;

  GetErrorStringParams gesp;
  GetErrorStringResults gesr;
  OpenDeviceParams oo;
  EstablishLinkParams egsp;
  EstablishLinkResults egsr;
  StartExposureParams segsp;
  float exposure_seconds;
  QueryCommandStatusParams qsgsp;
  QueryCommandStatusResults qsgsr;
  EndExposureParams eegsp;
  StartReadoutParams srgsp;
  ReadoutLineParams rlgsp;
  EndReadoutParams ergsp;
  GetDriverInfoParams gdp;
  GetDriverInfoResults0 gdr;

  fprintf(stderr, "Opening driver...\n");
  sts = SBIGUnivDrvCommand(CC_OPEN_DRIVER, NULL, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Opening driver done [%d]\n", sts);

  gdp.request = 0;
  sts = SBIGUnivDrvCommand(CC_GET_DRIVER_INFO, &gdp, &gdr);
  /* Version is a BCD representation XX.XX (cf. documentation) */
  fprintf(stderr, "name = [%s]  version = [%02x.%02x]  maxRequest=[%d]\n", 
	  gdr.name,
	  ((gdr.version & 0xFF00) >> 8),
	  (gdr.version & 0x00FF),
	  gdr.maxRequest);

  fprintf(stderr, "Opening device...\n");
  oo.deviceType = 0x7F00;
  oo.lptBaseAddress = 0x00;
  oo.ipAddress = 0;
  sts = SBIGUnivDrvCommand(CC_OPEN_DEVICE, &oo, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Opening device done [%d]\n", sts);

  fprintf(stderr, "Establishing link...\n");
  ok = 0;
  for(i=1 ;i<=5;i++) {
    egsp.sbigUseOnly = 0;
    fprintf(stderr, "Establishing link... try %d...\n", i);
    sts = SBIGUnivDrvCommand(CC_ESTABLISH_LINK, &egsp, &egsr);
    if (sts) {
      gesp.errorNo = sts;
      SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
      fprintf(stderr, "[%s]\n", gesr.errorString);
    }
    if (sts == 0) {
      ok = 1;
      break;
    }
  }
  if (!ok) {
    fprintf(stderr, "Establishing link failed [%d]. Stop.\n", sts);
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
    exit(10);
  }
  fprintf(stderr, "Establishing link done [%d]\n", sts);

  exposure_seconds = 5.0;
  segsp.ccd = CCD_IMAGING;
  segsp.abgState = ABG_LOW7;
  segsp.openShutter = SC_OPEN_SHUTTER;
  segsp.exposureTime = (long)floor(exposure_seconds * 100);
  sts = SBIGUnivDrvCommand(CC_START_EXPOSURE, &segsp, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Starting exposure done [%d]\n", sts);

  qsgsr.status = 0;
  do {
    sleep(1);
    qsgsp.command = CC_START_EXPOSURE;
    sts = SBIGUnivDrvCommand(CC_QUERY_COMMAND_STATUS, &qsgsp, &qsgsr);
    fprintf(stderr, "Status = %d\n", qsgsr.status);
  } while (qsgsr.status != CE_NO_EXPOSURE_IN_PROGRESS) ;


  fprintf(stderr, "Ending exposure...\n");
  eegsp.ccd = CCD_IMAGING;
  sts = SBIGUnivDrvCommand(CC_END_EXPOSURE, &eegsp, NULL);
  fprintf(stderr, "Ending exposure done [%d]\n", sts);

  fprintf(stderr, "Starting readout...\n");
  srgsp.ccd = CCD_IMAGING;
  srgsp.readoutMode = 0;
  srgsp.top = 0;
  srgsp.left = 0;
  srgsp.height = HEIGHT;
  srgsp.width = WIDTH;
  sts = SBIGUnivDrvCommand(CC_START_READOUT, &srgsp, NULL);
  fprintf(stderr, "Starting readout done [%d]\n", sts);


  rlgsp.ccd = CCD_IMAGING;
  rlgsp.readoutMode = 0;
  rlgsp.pixelStart = 0;
  rlgsp.pixelLength = WIDTH;
  for(row=0; row<HEIGHT; row++) {
    /* initialize to zero the row for debug purposes */
    for (j=0; j<WIDTH; j++) *(img+row*WIDTH+j) = 0;
    /* reading the row */
    sts = SBIGUnivDrvCommand(CC_READOUT_LINE, &rlgsp, (img+row*WIDTH));
    if (sts) {
      fprintf(stderr, "Reading row [%d] error code [%d]\n", row, sts);
      gesp.errorNo = sts;
      SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
      fprintf(stderr, "[%s]\n", gesr.errorString);
    }
  }

  fprintf(stderr, "Ending readout...\n");
  ergsp.ccd = CCD_IMAGING;
  sts = SBIGUnivDrvCommand(CC_END_READOUT, &ergsp, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Ending readout done [%d]\n", sts);

  fprintf(stderr, "Closing device...\n");
  sts = SBIGUnivDrvCommand(CC_CLOSE_DEVICE, NULL, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Closing device done [%d]\n", sts);

  fprintf(stderr, "Closing driver...\n");
  sts = SBIGUnivDrvCommand(CC_CLOSE_DRIVER, NULL, NULL);
  if (sts) {
    gesp.errorNo = sts;
    SBIGUnivDrvCommand(CC_GET_ERROR_STRING, &gesp, &gesr);
    fprintf(stderr, "[%s]\n", gesr.errorString);
  }
  fprintf(stderr, "Closing driver done [%d]\n", sts);

  fprintf(stderr, "Saving the image into a FITS file...\n");
  status = 0;
  fits_create_file(&fptr, "!out.fits", &status);
  fits_report_error(stderr, status);
  fits_create_img(fptr, SHORT_IMG, naxis, naxes, &status);
  fits_report_error(stderr, status);
  fits_write_img(fptr, TSHORT, firstpixel, WIDTH*HEIGHT, img, &status);
  fits_report_error(stderr, status);
  fits_close_file(fptr, &status);
  fits_report_error(stderr, status);
  fprintf(stderr, "Saving the image into a FITS file done\n");

  return 0;
}
